//package driver_system;

import java.awt.Image;
import java.awt.image.BufferedImage;
import java.awt.image.IndexColorModel;
import java.beans.Encoder;
import java.io.BufferedInputStream;
import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.net.HttpURLConnection;
import java.net.MalformedURLException;
import java.net.URL;
import java.net.URLEncoder;
import java.lang.Long;

import javax.imageio.ImageIO;


public class DecodeAndPlayVideo implements Runnable {
    
    private String urlString, user, pass;
    private byte[] curFrame;
    private boolean frameAvailable;
    private boolean frameStarted;
    private Thread streamReader;
    private HttpURLConnection conn;
    private BufferedInputStream httpIn;
    private ByteArrayOutputStream jpgOut;
    private BufferedImage currentImage;
    private String status;
    
    public final static String VERSION = "0.1.0";
    public DecodeAndPlayVideo( String urlString, String user, String pass) {
        
        
        this.urlString = urlString;
        this.user = user;
        this.pass = pass;
        this.curFrame = new byte[0];
        this.frameAvailable = false;
        this.streamReader = new Thread(this, "HTTP Stream reader");
    }
    
    public DecodeAndPlayVideo() {
        // this.urlString = url;
        if(this.urlString.length()<3){
            System.err.println(status = "url sorgente stream mancante");
            return;
        }else{
            this.user = "";
            this.pass = "";
            this.curFrame = new byte[0];
            this.frameAvailable = false;
            this.streamReader = new Thread(this, "HTTP Stream reader");
        }
        
    }
    
    public boolean isAvailable() {
        return frameAvailable;
    }
    
    public void start() {
        streamReader.start();
    }
    public String getStatusString(){
        return this.status;
    }
    
    public void stop() {
        try {
            jpgOut.close();
            httpIn.close();
        }
        catch (IOException e) {
            System.err.println("Error closing streams: " + e.getMessage());
        }
        conn.disconnect();
    }
    
    public void dispose() {
        stop();
    }
    public long onlineTime(){
        return conn.getDate();
    }
    public boolean workingConnection(){
        try{
        if(this.conn.getResponseCode() == 200)
            return true;
        return false;
        }catch(IOException e){
            e.printStackTrace();
            return false;
        }
    }
    
    public void run() {
        URL url;
        URLEncoder urlEncod = null;
        System.err.println("inside run of decodeand play");
        // Base64Encoder base64 = new Base64Encoder();
        
        try {
            url = new URL(urlString);
        }
        catch (MalformedURLException e) {
            System.err.println(status = "Invalid URL");
            return;
        }
        
        try {
            conn = (HttpURLConnection)url.openConnection();
            System.out.println(conn.toString()+ Long.toString(onlineTime()));
            conn.connect();
            //conn.setRequestProperty("Authorization", "Basic " + urlEncod.encode(user + ":" + pass));
            httpIn = new BufferedInputStream(conn.getInputStream());
            
        }
        catch (IOException e) {
            status = "unable to connect";
            System.err.println("Unable to connect: " + e.getMessage());
            return;
        }
        
        try {
            httpIn = new BufferedInputStream(conn.getInputStream(), 8192);
            jpgOut = new ByteArrayOutputStream(8192);
        }
        catch (IOException e) {
            status = "I/O stream problem";
            System.err.println("Unable to open I/O streams: " + e.getMessage());
            return;
        }
        
        int prev = 0;
        int cur = 0;
        
        try {
            while ((cur = httpIn.read()) >= 0) {
                if (prev == 0xFF && cur == 0xD8) {
                    frameStarted = true;
                    jpgOut.close();
                    jpgOut = new ByteArrayOutputStream(8192);
                    jpgOut.write((byte)prev);
                }
                if (frameStarted) {
                    jpgOut.write((byte)cur);
                    if (prev == 0xFF && cur == 0xD9) {
                        curFrame = jpgOut.toByteArray();
                        frameStarted = false;
                        frameAvailable = true;
                        this.read();
                    }
                }
                prev = cur;
            }
            
        }catch (IOException e) {
            System.err.println("I/O Error: " + e.getMessage());
        }
    }
    
     private synchronized void read() {
        try {
            Thread.yield();
            ByteArrayInputStream jpgIn = new ByteArrayInputStream(curFrame);
            BufferedImage bufImg = ImageIO.read(jpgIn);
            jpgIn.close();
            //System.out.println(bufImg.toString());
            status = "connected";
            int w = bufImg.getWidth();
            int h = bufImg.getHeight();
            /* if (w != this.width || h != this.height) {
             this.resize(bufImg.getWidth(),bufImg.getHeight());
             }  //  TODO adattare la finestra alle dimensioni del frame*/
            //      bufImg.getRGB(0, 0, w, h, this.pixels, 0, w); // TODO Codifica nello spazio RGB
            BufferedImage tmpImg = new BufferedImage(w, h,BufferedImage.TYPE_3BYTE_BGR);
            tmpImg.getGraphics().drawImage(bufImg, 0, 0, null);
            this.currentImage = tmpImg; //TODO NEW Image
            
            //System.out.println("current is:"+this.currentImage);
            
            frameAvailable = false;
        }
        catch (IOException e) {
            status = "error on frame";
            System.err.println("Error acquiring the frame: " + e.getMessage());
        }
    }
    
    public void setSource(String string) {
        // TODO Auto-generated method stub
        this.urlString = string;
        
    }
    
    public synchronized BufferedImage getCurrentImage() { //sync
        // TODO Auto-generated method stub
        System.out.println("tryng to get current image");
        return this.currentImage;
    }
}
