//package driver_system;

import java.awt.Color;
import java.awt.Graphics2D;
import java.awt.event.KeyEvent;
import java.awt.image.BufferedImage;
import java.io.IOException;
import java.net.URL;
import java.util.Random;
import java.util.logging.Level;
import java.util.logging.Logger;
import javax.imageio.ImageIO;

/**
 *
 * @author s
 */

public class PlayerDrone
{
    
    private NetworkConnectionToRouter toDroneConnection = new NetworkConnectionToRouter();
    
    /**
     * X coordinate for drawing acceleration indicator.
     */
    public int x;
    /**
     * Y coordinate for drawing acceleration indicator.
     */
    public int y;
    /**
     * X coordinate for drawing status icon.
     */
    public int x_status = (int)(Framework.frameWidth * 0.01);
    /**
     * Y coordinate for drawing status icon.
     */
    public int y_status = (int)(Framework.frameHeight * 0.85);
    
    /**
     * Is rocket landed?
     */
    public boolean landed;
    
    /**
     * Has rocket critical?
     */
    public boolean critical;
    
    /**
     * Accelerating speed of the rocket.
     */
    private int speedAccelerating;
    /**
     * Stopping/ speed of drone. .
     */
    private int speedStopping;
    
    /**
     * Maximum speed that rocket can reach.
     */
    public int topSpeed;
    
    /**
     * How fast is running right motor
     */
    public int speedL;
    /**
     * How fast is running left motor
     */
    public int speedR;
    /*
     *Extra controls value
     *
     */
    public int cam_y;
    public int cam_x;
    public int orizon;
    
    
    
    /**
     * Image of the rocket in air.
     */
    private BufferedImage droneScheme;
    /**
     * Image of the rocket when landed.
     */
    private BufferedImage droneLandedImg;
    /**
     * Image of the rocket when critical.
     */
    private BufferedImage droneCrashedImg;
    
    
    /**
     * Width of rocket.
     */
    public int rocketImgWidth;
    /**
     * Height of rocket.
     */
    public int rocketImgHeight;
    
    
    public PlayerDrone()
    {
        
        System.out.println("costruendo nuova istanza di playerDrone");
        Initialize();
        LoadContent();
        
        
        
    }
    
    
    private void Initialize()
    {
        
        
        ResetPlayer();
        
        speedAccelerating = 2;
        speedStopping = 2;
        
        topSpeed = 200;
        
        
    }
    
    private void LoadContent()
    {
        try
        {
            URL rocketImgUrl = this.getClass().getResource("/resources/accelerom.png");
            droneScheme = ImageIO.read(rocketImgUrl);
            rocketImgWidth = droneScheme.getWidth();
            rocketImgHeight = droneScheme.getHeight();
            
            URL rocketLandedImgUrl = this.getClass().getResource("/resources/system_pause.png");
            droneLandedImg = ImageIO.read(rocketLandedImgUrl);
            
            URL rocketCrashedImgUrl = this.getClass().getResource("/resources/system_crytical.png");
            droneCrashedImg = ImageIO.read(rocketCrashedImgUrl);
            
            
        }
        catch (IOException ex) {
            Logger.getLogger(PlayerDrone.class.getName()).log(Level.SEVERE, null, ex);
        }
    }
    
    /**
     * Here we set up the rocket when we starting a new game.
     */
    public void ResetPlayer()
    {
        landed = false;
        critical = false;
        
        x = (int)(Framework.frameWidth * 0.5);//coordinate of the bitmaps
        y = (int)(Framework.frameHeight * 0.5);;
        
        speedL = 0;
        speedR = 0;
        if(motorsWorking()){//try catch(NullPointerException e)
            this.toDroneConnection.printRemote("reset rover string");
        }
    }
    
    
    /**
     * Here we move the rocket.
     */
    public void updatePlayer()
    {
        /*sovradimensioniamo le possibili combinazioni di tasti
         * per controlli aggiuntivi possibili:
         *su e giù -->assetto
         *w,d e 'a' --> pan tilt camera
         */
        
        // Se voglio andare a destra accellero il motore di sinistra e viceversa.
        //due motori solo avanti
        if(Canvas.keyboardKeyState(KeyEvent.VK_LEFT))
            speedR += speedAccelerating;
        else
            if(speedR > 0)
                speedR -= speedStopping; //to tare
        
        
        if(Canvas.keyboardKeyState(KeyEvent.VK_RIGHT))
            speedL += speedAccelerating;
        else
            if(speedL > 0)
                speedL -= speedStopping;
        
        //assetto avanti e indietro
        if(Canvas.keyboardKeyState(KeyEvent.VK_UP))
            orizon += speedAccelerating;
        else
            if(orizon > 0)
                orizon -= speedStopping;
        if(Canvas.keyboardKeyState(KeyEvent.VK_DOWN))
            orizon -= speedAccelerating;
        else
            if(orizon < 0)
                orizon += speedStopping;
        
        // Calculating position for moving then stopping in place.
        if(Canvas.keyboardKeyState(KeyEvent.VK_W))
            cam_y += speedAccelerating;
        if(Canvas.keyboardKeyState(KeyEvent.VK_S))
            cam_y -=speedStopping;
        if(Canvas.keyboardKeyState(KeyEvent.VK_A))
            cam_x -=speedStopping;
        if(Canvas.keyboardKeyState(KeyEvent.VK_D))
            cam_x +=speedAccelerating;
        
        
        // Moves the accellerom.
        x =(int)(Framework.frameWidth * 0.5)+(speedR-speedL);
        
        if(motorsWorking()){
            toDroneConnection.printRemote("printing current speed remotely");
            toDroneConnection.printRemote("L:"+speedL+"-R:"+speedR);
        }
        System.out.println("currents speed in PlayerDrone"+speedL+" "+speedR);
    }
    
    public void Draw(Graphics2D g2d)
    {
        System.out.println("into super draw of PlayerDrone");
        g2d.setColor(Color.white);
        if(this.toDroneConnection.getRouterStatus() == 200){
            g2d.drawString("rover connection status ok",(int)(Framework.frameWidth * 0.80),(int)(Framework.frameHeight * 0.90));
        }else {
            g2d.drawString("rover connection error",(int)(Framework.frameWidth * 0.80),(int)(Framework.frameHeight * 0.90));
            this.critical = true;
        }
        // If the rocket is landed.
        if(landed)
        {
            System.out.println("as landed");
            g2d.drawImage(droneLandedImg, x_status, y_status, null);
        }
        // If the rocket is critical.
        else {
            if(critical){
            System.out.println("as critical");
        
            //g2d.drawImage(droneCrashedImg, x, y + rocketImgHeight - droneCrashedImg.getHeight(), null);
            g2d.drawImage(droneCrashedImg, x_status, y_status, null);
            
        }
            // If the rocket is still in the space.
        else
        {
            System.out.println("tryng to paint accellerom in x: "+x);
            g2d.drawImage(droneScheme, x, y, null);
            /*repaint in place the drone schema
             bisogna troncare i valori a 200
             ricalcolare la y su 'orizon'
             g2d.rotate(Math.asin(this.speedY/Math.sqrt(Math.pow(this.speedX,2) + Math.pow(this.speedY, 2))));
             */
        }
        }
    }
    
    
    public boolean motorsWorking() {
        // TODO Auto-generated method stub
    return this.toDroneConnection.isConnected();
        
    }
    
}
